import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0
import "Controller.js" as Controller
import IconTheme 1.0
Rectangle {
    id : appArea
    width : 100
    height : 100
    color : "transparent"
    property ListModel appModel : null
    property var rootRect : null
    property var appAreaController  : null

    ScrollView{
        id : scrollArea
        width : parent.width
        height : parent.height
        property var flowRows : 0
        contentWidth : parent.width
        contentHeight : flowRows * ((rootRect.height / 22 * 5) + 12 + 40) > parent.height ? flowRows * ((rootRect.height / 22 * 5) + 12 + 40) : parent.height
        ScrollBar.horizontal.policy : ScrollBar.AlwaysOff
        ScrollBar.vertical.policy : ScrollBar.AlwaysOff
        clip : true
        MouseArea{
            anchors.fill : parent
            onClicked : {
                appAreaController.closeMultitaskView()
            }
        }

        Flow{
            id : flowLayout
            spacing : 0
            width : screenRect.width
            flow : Flow.LeftToRight

            move :Transition { //子项增加时或者减少时的过度，也就是子项的visible变化的过度
                NumberAnimation {
                    properties : "x,y";
                    easing.type : Easing.InOutQuad;
                    duration : 500
                }
            }

            Repeater{
                property int cnt : 0
                id : rep
                model : appModel
                function adjustSize(){
                    // 计算当前布局的方式
                    var sqrtCnt = Math.ceil(Math.sqrt(rep.cnt));
                    var cols = sqrtCnt;
                    if (cols > 3) {
                        cols = 3;
                    }

                    var lineWidth = rootRect.width;
                    // 调整矩形框的大小
                    for(var i = 0; i < rep.cnt; i += cols)
                    {
                        var blockWidth = Math.floor(lineWidth / Math.min(cols, rep.cnt - i));
                        for(var j = 0;j < Math.min(cols, rep.cnt - i); ++j)
                        {
                            if(itemAt(i + j)){
                                itemAt(i + j).width = blockWidth
                            }
                        }
                    }
                    scrollArea.flowRows = Math.ceil(rep.cnt/cols)
                }

                onItemAdded : {
                    cnt++;
                    adjustSize();
                }

                onItemRemoved : {
                    cnt--;
                    adjustSize();
                }

                Rectangle{
                    id : thumbnailitem
                    height : (rootRect.height / 22 * 5) + 12 + 40
                    color : "transparent"
                    radius : 14
                    function thumbwidth(){
                        var max_height = rootRect.height / 22 * 5 - 12
                        var max_width = rootRect.width / 3 - 12
                        var oriW = model.width;
                        var oriH = model.height;
                        var dh, dw;
                        var isfind = 0;
                        // 情况2
                        dh = max_height
                        dw = oriW * dh / oriH;
                        if(dw <= max_width){
                            isfind = 1;
                        }
                        // 情况1
                        if(!isfind){
                            dw = max_width
                            dh = oriH * max_width / oriW
                        }
                        return dw
                    }

                    function thumbheight(){
                        var max_height = rootRect.height / 22 * 5 - 12
                        var max_width = rootRect.width / 3 - 12
                        var oriW = model.width;
                        var oriH = model.height;
                        var dh, dw;

                        var isfind = 0;
                        // 情况2
                        dh = max_height
                        dw = oriW * dh / oriH
                        if(dw <= max_width){
                            isfind = 1;
                        }
                        // 情况1
                        if(!isfind){
                            dw = max_width
                            dh = oriH * max_width / oriW
                        }
                        return dh
                    }

                    Rectangle {
                        id : appWindowAnchor
                        anchors.centerIn : parent
                        width : thumbnailitem.thumbwidth() + 12 > rootRect.width / 10 ? thumbnailitem.thumbwidth() + 12  : rootRect.width / 10
                        height : parent.height
                        color : "transparent"

                        PreviewWindow {
                            id : appWindow
                            width : thumbnailitem.thumbwidth() + 12 > rootRect.width / 10 ? thumbnailitem.thumbwidth() + 12  : rootRect.width / 10
                            height : (rootRect.height/22*5) + 12 + 40
                            previewAreaWidth : thumbnailitem.thumbwidth() + 12 > rootRect.width / 10 ? thumbnailitem.thumbwidth() + 12  : rootRect.width / 10
                            previewAreaHeight :  rootRect.height / 22 * 5
                            windowType : "app"
                            controller : new Controller.AppController(multitaskViewModel, appWindow, multitaskViewManager, ["none"]);

                            onNowDragActiveChanged : {
                                if(!appWindow.winowDragActive) {
                                    appWindow.zoomOut()
                                    var fromPos  = appWindowBack.mapToItem(appWindowAnchor,0,0)

                                    appWindow.posX = fromPos.x
                                    appWindow.posY = fromPos.y
                                    appAreaController.onSourceDropped(appWindow,allroot.highlightDesktop,allroot.highlightDesktop)
                                } else {
                                    appWindow.zoomIn()
                                }
                            }

                            states :  [
                                State {
                                    name : "default"
                                    PropertyChanges { target : appWindow; borderWidth : 0 }
                                    PropertyChanges { target : appWindow; closeButtonVisble : false }
                                },

                                State {
                                    name : "hover"
                                    PropertyChanges { target : appWindow; borderWidth : 4 }
                                    PropertyChanges { target : appWindow; closeButtonVisble : true }
                                }
                            ]

                            Rectangle {
                                id : appWindowBack
                                anchors.horizontalCenter : appWindow.horizontalCenter
                                y : 40+5
                                height : appWindow.previewAreaHeight-12
                                width : appWindow.previewAreaWidth-12
                                color : Qt.rgba(255,255,255, 0.5)
                                clip : true
                                visible: true
                                layer.enabled : multitaskViewManager.useOpenGL() ? true : false
                                layer.effect : OpacityMask{
                                    maskSource : Rectangle{
                                        height : appWindow.previewAreaHeight-12
                                        width : appWindow.previewAreaWidth-12
                                        radius : 8
                                    }
                                }

                                KWinComponents.WindowThumbnailItem{
                                    id : tmp
                                    anchors.centerIn : parent
                                    z: appWindow.onTop ? 100 : 20
                                    height : thumbnailitem.thumbheight()
                                    width : thumbnailitem.thumbwidth()
                                    wId: model.uuid
                                    clip : true
                                }
                            }

                            Component.onCompleted : {
                                appWindow.setCaption(model.caption)
                                appWindow.setId(model.uuid)
                            }
                        }
                    }
                }//Rectangle
            }//Repeater
        }//Flow
    }//ScrollView
}
