import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0
import "TabletController.js" as TabletController
import IconTheme 1.0


Rectangle{
    id : tabletScrollViewArea
    width : 1920
    height : 1080
    color : "transparent"
    property ListModel appModel : null
    property var rootRect : null
    //调用全部清除按钮动效的时候需要用到
    property var appAreaController  : null

    ScrollView{
        id : scrollView

        //更新滚动区域的大小，之所以这么做，是为了解决滚动区域增加鼠标事件后，滚动失效的问题
        function getContentWidth()
        {
            var maxColumn = Math.ceil(appModel.count / 2)
            var scrollViewContentWidth = appArea.width
            if (appModel.count <= 4)
            {
                scrollViewContentWidth = appArea.width
                return Math.floor(scrollViewContentWidth)
            }
            scrollViewContentWidth = maxColumn * preWinWidth + maxColumn * gridLayout.columnSpacing + 2 * gridLayout.leftPadding
            return Math.floor(scrollViewContentWidth)
        }

        x : 0
        y : 0
        contentWidth : Math.floor(scrollView.getContentWidth())
        contentHeight : Math.floor(appArea.height)
        width : Math.floor(rootRect.width)
        ScrollBar.horizontal.policy : ScrollBar.AlwaysOff
        ScrollBar.vertical.policy : ScrollBar.AlwaysOff

        MouseArea{
            anchors.fill : parent
            propagateComposedEvents: true

            onClicked: {
                mouse.accepted = false
            }
        }

        Grid{
            id : gridLayout
            anchors.fill : parent
            anchors.centerIn : parent
            columnSpacing : Math.floor(preWinVecSpace)
            rowSpacing : Math.floor(preWinHorSpace)
            leftPadding : Math.floor(scrollViewLeftSpace)
            rightPadding : Math.floor(scrollViewLeftSpace)
            rows : 2
            flow : Grid.TopToBottom

            //显示多任务视图时，每个Item在显示的时候执行的动效
            add : Transition {
                id : displayPreviewWinAnimation

                ParallelAnimation{
                    NumberAnimation {
                        property : "opacity"
                        duration : 300
                        easing.type : Easing.InOutQuart
                        from : 0
                        to : 1
                    }
                   NumberAnimation {
                       property : "x"
                       duration :300
                       easing.type : Easing.InOutQuart
                       from : Math.floor(rootRect.width / 2 - 510/*窗口的宽度*/ / 2)
                   }
                   NumberAnimation {
                       property : "y"
                       duration : 300
                       easing.type : Easing.InOutQuart
                       from : Math.floor(rootRect.height)
                   }
                   NumberAnimation {
                       property : "scale"
                       from : 0
                       to : 1.0
                       duration : 150
                   }
                }

            }

            //关闭小窗口时，W补齐方式的动画
            move : Transition {
                NumberAnimation{
                    properties : "x,y";
                    easing.type : Easing.InOutQuart;
                    duration : 300
                }
            }


            Repeater{
                 id : previewWindowRepeater
                 model : appModel

                 Rectangle{
                     id : appWindowBack
                     height : Math.floor(preWinHeight + iconToThumbSpacing + iconAndCloseButtonHeight)
                     width : Math.floor(preWinWidth)
                     color : "transparent"
                     radius : 14

                     TabletPreviewWindow {
                         id : appWindow
                         height : Math.floor(preWinHeight + iconToThumbSpacing + iconAndCloseButtonHeight)
                         width :  Math.floor(preWinWidth)
                         color : "transparent"
                         radius : 14
                         premodel : appModel

                         states :  [
                             State {
                                 name : "default"
                                 PropertyChanges { target : appWindow; borderWidth : 0 }
                                 PropertyChanges { target : appWindow; closeButtonVisble : false }
                             },

                             State {
                                 name : "hover"
                                 PropertyChanges { target : appWindow; borderWidth : 4 }
                                 PropertyChanges { target : appWindow; closeButtonVisble : true }
                             }
                         ]

                         Connections{
                             target : rootView
                             onQmlViewSpaceClicked:{
                                 appWindow.appController.startSpaceClickedPreWindowAnimation()
                             }

                             onQmlClearAllAppWindow:{
                                 appWindow.appController.startClearAllWindowAnimation()
                             }
                         }

                         Component.onCompleted: {
                             appWindow.setCaption(model.caption)
                             appWindow.setId(model.uuid)
                             appWindow.setPreViewNoScaledHeight(model.height)
                             appWindow.setPreViewNoScaledWidth(model.width)
                             appWindow.setPreViewNoScaledXCoordinate(model.x)
                             appWindow.setPreViewNoScaledYCoordinate(model.y)
                         }
                     }
                 }
            }
        }
    }
}
