import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import IconTheme 1.0
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0

Rectangle{
    id : tabletViewMainWindow

    property var controller : new TabletController.Controller(multitaskViewModel, tabletViewMainWindow, multitaskViewManager, ["none"])
    Keys.onPressed: {
       if (event.key === Qt.Key_Escape) {
           controller.closeMultitaskView();
           return;
       }
       if (event.modifiers & Qt.MetaModifier) {
           if (event.key === Qt.Key_Tab) {
               controller.closeMultitaskView();
               return;
           }
       }
    }

    signal qmlAppWindowActivated()
    signal qmlAppWindowCreated(int id)
    signal qmlAppWindowRemoved(int id)
    signal qmlDeviceModeChanged(bool modeFlag)
    signal qmlOrientationChanged(string orientation)

    //后台发送qmlAppWindowRemoved，通过qmlAppWindowRemovedForward通知视图模块删除model内的数据
    signal qmlAppWindowRemovedForward(int id)

    Connections{
        target : tabletViewMainWindow

        // onQmlAppWindowActivated: {
        //     controller.closeMultitaskView();
        // }

        onQmlAppWindowCreated: {
            controller.closeMultitaskView();
        }

        onQmlDeviceModeChanged : {
           controller.closeMultitaskView();
        }

        onQmlAppWindowRemoved: {
            qmlAppWindowRemovedForward(id)
        }
    }

    property var screenIndex : 0
    property var screenRect : multitaskViewModel.getFullScreenGeometry(1,0)
    property string  orientation: multitaskViewModel.rotationMode
    //整体缩放比
    property var tabletScaleFactor : screenRect.width > screenRect.height ? screenRect.height / 1080 : screenRect.width / 1080
    //预览窗口宽高比
    property var preWinAspectRatio: screenRect.width / screenRect.height
    //图标和缩略图的间距
    property var iconToThumbSpacing : Math.floor(18 * tabletScaleFactor)
    //图标高度
    property var iconAndCloseButtonHeight : Math.floor(44 * tabletScaleFactor)
    //缩略图高度
    property var preWinHeight : Math.floor(320 * tabletScaleFactor)
    //缩略图宽度
    property var preWinWidth : Math.floor(preWinHeight * preWinAspectRatio)
    //预览窗口列间距
    property var preWinVecSpace : Math.floor(112 * tabletScaleFactor)
    //预览窗口行间距
    property var preWinHorSpace : Math.floor(56 * tabletScaleFactor)
    //滚动区域距离左侧和右侧的间距
    property var scrollViewLeftSpace : Math.floor(112 * tabletScaleFactor)
    //滚动区域高度
    property var scrollViewHeight : Math.floor(820 * tabletScaleFactor)
    //全部清除按钮x
    property var clearWindwoX : Math.floor(747 * tabletScaleFactor)
    //全部清除按钮y
    property var clearWindwoY : Math.floor(994 * tabletScaleFactor)
    //滚动区域y轴起始高度
    property var scrollViewYStartPoint : Math.floor(getCurrentPreWindowHeight())

    width : Math.floor(screenRect.width)
    height : Math.floor(screenRect.height)
    x : Math.floor(screenRect.x)
    y : Math.floor(screenRect.y)
    color : "transparent"

    /**********************************************函数*********************************************/
    function getCurrentPreWindowHeight()
    {
        if ( orientation == "normal" ||  orientation == "upside-down" )
        {
            scrollViewYStartPoint = Math.floor(99 * tabletScaleFactor)
            preWinHeight = Math.floor(320 * tabletScaleFactor)
            tabletParentWindow.preWinWidth = Math.floor(preWinHeight * preWinAspectRatio)
            preWinVecSpace = Math.floor(112 * tabletScaleFactor)
            preWinHorSpace = Math.floor(56 * tabletScaleFactor)
            scrollViewLeftSpace = Math.floor(112 * tabletScaleFactor)
            scrollViewHeight = Math.floor(820 * tabletScaleFactor)
            clearWindwoX = Math.floor(747 * tabletScaleFactor)
            clearWindwoY = Math.floor(994 * tabletScaleFactor)
        }
        else if ( orientation == "left" ||  orientation == "right" )
        {
            scrollViewYStartPoint = Math.floor(196 * tabletScaleFactor)
            preWinHeight = Math.floor(498 * tabletScaleFactor)
            preWinWidth = Math.floor(preWinHeight * preWinAspectRatio)
            preWinVecSpace = Math.floor(96 * tabletScaleFactor)
            preWinHorSpace = Math.floor(68 * tabletScaleFactor)
            scrollViewLeftSpace = Math.floor(96 * tabletScaleFactor)
            scrollViewHeight = Math.floor(1196 * tabletScaleFactor)
            clearWindwoX = Math.floor(477 * tabletScaleFactor)
            clearWindwoY = Math.floor(1534 * tabletScaleFactor)
        }
    }

    TabletMultitaskView {
        id : tabletViewMain
        anchors.fill : parent
        color : "transparent"
    }
}
