import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import IconTheme 1.0
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0

Rectangle{
    id : tabletRootWindow
    anchors.fill : parent
    property var controller : new TabletController.Controller(multitaskViewModel, tabletRootWindow, multitaskViewManager, ["none"])
    property var appWindowCount : 0
    property var loadedWindowNum : 0
    property ListModel tabletViewRootModel : ListModel{}

    /**********************************************函数*********************************************/
    function viewSpacingClickedController()
    {
        clearAllWindow.startClearAllBtnDisappearAni()
        qmlViewSpaceClicked()
    }

    function startNoWindowTipsAnimation()
    {
        noWindowTipsAnimation.start()
    }
    function startSequentialLoadWindowTimer()
    {
        sequentialLoadWindowTimer.start()
    }

    function fetchWindowNum(){
        var appWindowList =  multitaskViewModel.getAppWindowList(1,tabletViewMainWindow.screenIndex)
        for (var i in appWindowList)
        {
            tabletRootWindow.appWindowCount = tabletRootWindow.appWindowCount+1
        }
        controller.initViewAnimation()
    }

    function sequentialLoadWindow() {
        var appWindowList =  multitaskViewModel.getAppWindowList(1,tabletViewMainWindow.screenIndex)
        if (loadedWindowNum < appWindowCount && loadedWindowNum < 6 )
        {
            var info = appWindowList[loadedWindowNum]
            tabletViewRootModel.append({"x": info.x, "y": info.y,
                              "width": info.width, "height": info.height,
                              "id": info.id,"minimized":info.minimized, "caption": info.caption,
                              "uuid" : String(info.uuid)});
            loadedWindowNum = loadedWindowNum+1
            console.log("加载窗口中，目前已加载窗口数量为" + loadedWindowNum)

        }
        else
        {
            sequentialLoadWindowTimer.stop()
            console.log("计时器停止，窗口数量为： " + tabletViewRootModel.count)
        }
        if (loadedWindowNum >= 6)
        {
            sequentialLoadWindowTimer.stop()
            while(loadedWindowNum <  appWindowCount)
            {
                var info = appWindowList[loadedWindowNum]
                tabletViewRootModel.append({"x": info.x, "y": info.y,
                                   "width": info.width, "height": info.height,
                                   "id": info.id,"minimized":info.minimized, "caption": info.caption,
                                   "uuid" : String(info.uuid)});
                loadedWindowNum = loadedWindowNum + 1
            }
            console.log("sequentialLoadWindow的剩余窗口加载完毕！！！")

        }
        if (loadedWindowNum >= appWindowCount || appWindowCount <= 0)
        {
            console.log("全部清除按钮显示或者无窗口显示")
            sequentialLoadWindowTimer.stop()
            clearAllWindow.startClearAllBtnAppearAni()
        }
    }

    /**********************************************动画*********************************************/
    SequentialAnimation{
        id :  noWindowTipsAnimation
        //无窗口文字出现动画
        NumberAnimation {
            id : noWindowTipsAppearAni
            target : noWindowText
            property : "opacity"
            duration : 1000
            easing.type : Easing.InOutQuart
            from : 0
            to : 1
        }

        //无窗口文字消失动画
        NumberAnimation {
            id : noWindowTipsDisappearAni
            target : noWindowText
            property : "opacity"
            duration : 1000
            easing.type : Easing.InOutQuart
            from : 1
            to : 0
        }
        onFinished: {
           controller.closeMultitaskView()
        }
    }


    //控制启动动效中，小窗口之间的间隔
    Timer {
        id : sequentialLoadWindowTimer;
        interval : 100;
        repeat : true
        running : false
        triggeredOnStart : false
        onTriggered: {
            tabletRootWindow.sequentialLoadWindow()
          }
    }

    //这里放置后台提供的桌面背景的Item
    KWinComponents.DesktopBackground {
        width : Math.floor(tabletRootWindow.width)
        height : Math.floor(tabletRootWindow.height)
        blurAlpha : 0.7
        z : -1
    }

    //无应用时的窗口显示
    Text {
        id : noWindowText
        anchors.centerIn : parent
        font.bold : true
        font.pixelSize : 25
        color : "white"
        text : qsTr("无应用开启")
        visible : tabletViewRootModel.count == 0
        opacity : 0
    }

    //根窗口的鼠标事件
    MouseArea{
        anchors.fill : parent
        onClicked: {
            controller.viewSpacingClickedController()
        }
    }

    Component.onCompleted: {
        tabletRootWindow.fetchWindowNum();
    }
}
